package rition.service.panel;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import rition.common.data.dao.mapper.MetricRecordMapper;
import rition.common.data.entity.MetricRecordEntity;

import java.time.Instant;
import java.util.List;

@Slf4j
@Service
public class MetricService {

    private final MetricRecordMapper metricRecordMapper;

    public MetricService(MetricRecordMapper metricRecordMapper) {
        this.metricRecordMapper = metricRecordMapper;
    }

    private static final long HOUR = 3600 * 1000;
    private static final long DAY = 24 * 3600 * 1000;
    private static final long MONTH = 30L * 24 * 3600 * 1000;

    /**
     * 按时间范围获取监测数据
     *
     * @param instanceId  实例id
     * @param metricItems 监测指标项
     * @param start       开始时间
     * @param end         结束时间
     * @return 监测指标数据，指标数据为给定的指标平均统计
     */
    public List<MetricRecordEntity> getMetricDataRange(String instanceId,
                                                       List<String> metricItems,
                                                       Instant start, Instant end) {
        // 时间跨度小于1小时：不按时间粒度获取
        // 时间跨度大于1小时：时间粒度为1分钟
        // 时间跨度超过一天：时间粒度为小时
        var diff = end.minusMillis(start.toEpochMilli()).toEpochMilli();
        if (diff < HOUR) {
            var query = new LambdaQueryWrapper<MetricRecordEntity>()
                    .eq(MetricRecordEntity::getInstanceId, instanceId)
                    .between(MetricRecordEntity::getTime, start, end);
            return metricRecordMapper.selectList(query);
        } else if (diff < DAY) {
            return metricRecordMapper.getMetricDataGroupByMinute(instanceId, metricItems, start, end);
        } else {
            return metricRecordMapper.getMetricDataGroupByHour(instanceId, metricItems, start, end);
        }
    }
}
